'use strict';

const Types = require('paradox-utils').SchemaTypes;
const _ = require('lodash');
const __ = require('paradox-utils');

class Instance {
    constructor(conn, model, data, options) {
        this._conn = conn;
        this._fields = model;
        this.dataValues = data || {};
        this._options = options || {};
        this._readonly = this._options.fields && this._options.fields.length > 0;

        this.applyValues(data, true);
    }

    /* -------------------------------------------------------------------------
     * Utils
     * ---------------------------------------------------------------------- */

    applyValues(data, fullApply) {
        let fields = this._fields.getFields();

        if(data instanceof Instance) {
            data = data.getValues();
        }

        if(!_.isPlainObject(data)) {
            throw new Error('invalid data format in instance');
        }

        data = __.object.simplify(data);

        this.dataValues = data;

        for(let fieldName in fields) {
            // Extract the value
            if(fieldName in data) {
                this[fieldName] = this.formatValue(fields[fieldName], data[fieldName]);

            } else {
                if(fullApply) {
                    if('defaultValue' in fields[fieldName]) {
                        this[fieldName] = fields[fieldName].defaultValue;
                    }
                }
            }
        }

        return this;
    }

    getValues() {
        let fields = this._fields.getFields();
        let values = {};

        for(let fieldName in fields) {
            // Extract the value
            if(fieldName in this) {
                values[fieldName] = this[fieldName];

            } else {
                if('defaultValue' in fields[fieldName]) {
                    values[fieldName] = fields[fieldName].defaultValue;

                } else {
                    values[fieldName] = null;
                }
            }
        }

        return values;
    }

    formatValue(field, value) {
        // Date
        // if(field.type instanceof Types.DATE.class) {
        //     if(typeof value == 'string' && value) {
        //         value = new Date(value);
        //     }
        // }

        // Number
        if(field.type instanceof Types.NUMBER.class) {
            if(typeof value == 'string') {
                value = Number(value);
            }
        }

        return value;
    }

    // // TODO: Temporal
    // validate() {
    //
    // }

    /* -------------------------------------------------------------------------
     * Modify
     * ---------------------------------------------------------------------- */

    /**
     *
     * @returns {Promise}
     */
    save(options) {
        return this._fields.saveInstance(this, Object.assign(options || {}, this._options));
    }

    /**
     * @param data
     * @param options
     * @returns {Promise}
     */
    update(data, options) {
        this.applyValues(data);
        return this.save(Object.assign(options || {}, this._options));
    }

    /* -------------------------------------------------------------------------
     * Remove
     * ---------------------------------------------------------------------- */

    destroy(options) {
        return this._fields.destroyInstance(this, Object.assign(options || {}, this._options));
    }

    /* -------------------------------------------------------------------------
     * Utils
     * ---------------------------------------------------------------------- */
    reload(options) {
        return this._fields.reloadInstance(this, Object.assign(options || {}, this._options));
    }
}

module.exports = Instance;